const cache = require('persistent-cache');
const crypto = require('crypto');

let urlCache;

function getKey(url, options) {
  return crypto
    .createHash('md5')
    .update(url)
    .update(JSON.stringify(options))
    .digest('hex');
}

function cleanup() {
  if (urlCache) {
    urlCache.deleteSync();
  }
}

function checkCache(url, options, generateValue) {
  const dir = options && options.cacheDir;
  if (!dir) return generateValue();
  if (!urlCache) {
    urlCache = cache({
      base: dir,
      name: 'github_embed_cache',
      memory: true,
      persist: true,
    });
  }

  const key = getKey(url, options);
  const cachedValue = urlCache.getSync(key);
  if (cachedValue) {
    return cachedValue;
  }
  return generateValue().then((value) => {
    urlCache.putSync(key, value);
    return value;
  });
}

module.exports = {
  getKey,
  checkCache,
  cleanup,
};
